from zope.interface import implements, Interface, alsoProvides
from zope.component import adapts, getMultiAdapter

from Products.CMFCore.utils import getToolByName
from Products.IMS.config import SendMessageToMany
from Products.IMS.interfaces import IIMSMessage, ISentMessage, IReceivedMessage, ISendable

from AccessControl.SecurityManagement import newSecurityManager

class IMSMessage(object):
    """ Adapter to send IMS messages
    """
    implements(IIMSMessage)
    adapts(Interface)

    def __init__(self, context):
        self.context = context

    def _getSafeId(self, container, title):
        utils = getToolByName(self.context, 'plone_utils')
        id = utils.normalizeString(title)
        objids = container.objectIds()
        if id in objids:
            i = 2
            while '%s-%s' % (id, i) in objids: i += 1;
            id = '%s-%s' % (id, i)
        return id

    def _shortenLines(self, message):
        lines = message.split("\n")
        shortened = []
        for line in lines:
            if line.startswith('>') or not line:
                shortened.append(line)
            else:
                words = line.split(' ')
                if len(words):
                    sline = words[0]
                    del words[0]
                    for word in words:
                        if len('%s %s' % (sline, word)) > 65:
                            shortened.append(sline)
                            sline = word
                        else:
                            sline = '%s %s' % (sline, word)
                    shortened.append(sline)
        return '\n'.join(shortened)


    def _createMessage(self, container, title, message, receiver, sender, replyTo=None, interface=None):
        try:
            typestool = getToolByName(self.context, 'portal_types')
            mship = getToolByName(self.context, 'portal_membership')
            id = self._getSafeId(container, title)
            current_auth = mship.getAuthenticatedMember()
            newSecurityManager(self.context.REQUEST, container.getWrappedOwner())
            typestool.constructContent(type_name='Message', container=container, id=id, title=title, message=message, receiver=receiver, replyTo=replyTo, sender=sender)
            instance = container[id]
            if interface is not None:
                alsoProvides(instance, interface)
            instance.reindexObject()
            newSecurityManager(self.context.REQUEST, current_auth)
            return instance
        except:
            return None

    def _getMessageFolder(self, userid, id, type_name):
        typestool = getToolByName(self.context, 'portal_types')
        mship = getToolByName(self.context, 'portal_membership')
        current_auth = mship.getAuthenticatedMember()
        homefolder = mship.getHomeFolder(userid)
        if homefolder is None:
            mship.createMemberarea(userid)
            homefolder = mship.getHomeFolder(userid)
        if not ISendable.providedBy(homefolder):
            alsoProvides(homefolder, ISendable)
        if not id in homefolder.objectIds():
            newSecurityManager(self.context.REQUEST, homefolder.getWrappedOwner())
            typestool.constructContent(type_name=type_name, container=homefolder, id=id)
            folder = homefolder[id]
            folder.reindexObject()
            newSecurityManager(self.context.REQUEST, current_auth)
        return homefolder[id]

    def sendMessage(self, title, message, receiver, sender=None, replyToSender=None, replyToReceiver=None):
        """Send a IMS message
        """
        if sender is None:
            portal_state = getMultiAdapter((self.context, self.context.REQUEST), name=u'plone_portal_state')
            member = portal_state.member()
            sender = member.getId()

        message = self._shortenLines(message)

        if isinstance(receiver, str):
            receiver=[receiver]
        received = []
        for r in receiver:
            received.append(self._createMessage(self._getMessageFolder(r, 'received', 'ReceivedMessageFolder'), title, message, [r], sender, replyToReceiver, IReceivedMessage))
        if received is not None and sender:
            sent = self._createMessage(self._getMessageFolder(sender, 'sent', 'SentMessageFolder'), title, message, receiver, sender, replyToSender, ISentMessage)
            if sent is not None:
                for r in received:
                    r.setCompanion(sent)
                return sent
        else:
            return None

    def replyToMessage(self, instance, title, message):
        """Reply to a IMS message
        """
        sender = instance.getReceiver()[0]
        receiver = instance.getSender()
        replyTo = instance
        reply = self.sendMessage(title, message, receiver, sender, replyTo, instance.getCompanion())
        if reply is not None:
            instance.replied = True
            instance.reindexObject()
            return reply
        return False

    def forwardMessage(self, instance, title, message, receiver):
        """Forward a IMS message to somebody
        """
        forward = self.sendMessage(title, message, receiver, None, None)
        if forward is not None:
            instance.forwarded = True
            instance.reindexObject()
            return forward
        return None




from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.layout.viewlets.common import ViewletBase
from zope.component import getMultiAdapter

from Products.CMFCore.utils import getToolByName

from Products.IMS.interfaces import IReceivedMessage

class IMSBarViewlet(ViewletBase):
    template = ViewPageTemplateFile('templates/ims_bar.pt')

    def render(self):
        mship = getToolByName(self.context, 'portal_membership')
        if mship.isAnonymousUser():
            return ''
        
        return self.template()

    def update(self):
        mship = getToolByName(self.context, 'portal_membership')
        if mship.isAnonymousUser():
            return
        
        super(IMSBarViewlet, self).update()

        context_state = getMultiAdapter((self.context, self.request),
                                        name=u'plone_context_state')
        tools = getMultiAdapter((self.context, self.request), name=u'plone_tools')

        try: # Plone 4+
            self.ims_actions = context_state.actions(category="ims")
        except TypeError: # Plone 3
            self.ims_actions = context_state.actions().get('ims', ())
        for action in self.ims_actions:
            if action['id'] == 'receivedmessages':
                catalog = getToolByName(self.context, 'portal_catalog')
                portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
                member = portal_state.member()
                action['unread'] = len(catalog(object_provides=IReceivedMessage.__identifier__, 
                                               path={'query': '%s/received' % '/'.join(mship.getHomeFolder(member.getId()).getPhysicalPath())},
                                               read=False))

        plone_utils = getToolByName(self.context, 'plone_utils')
        self.getIconFor = plone_utils.getIconFor


from zope.interface import Interface
from zope.component.interfaces import IObjectEvent

class IMessageFolder(Interface):
    """ A folder holding messages
    """

class IReceivedMessageFolder(IMessageFolder):
    """ The folder holding received messages
    """

class ISentMessageFolder(IMessageFolder):
    """ The folder holding sent messages
    """

class ISendable(Interface):
    """ A object which id is the id of a user to which a message may be sent
    """

class IMessage(Interface):
    """ A message
    """

    def reply(self, message, request):
        """ Reply to this message
        """

    def forward(self, message, receiver, request):
        """ Forward this message to somebody
        """

class ISentMessage(IMessage):
    """ A sent message
    """

class IReceivedMessage(IMessage):
    """ A received message
    """

class IIMSMessage(Interface):
    """ An adapter for the BrowserRequest to handle IMS messages.
    """

    def sendMessage(title, message, receiver, sender=None, replyTo=None):
        """ Send a IMS message.
        """

    def replyToMessage(instance, message):
        """ Reply to a IMS message
        """

    def forwardMessage(instance, message, receiver):
        """ Forward a IMS message to somebody
        """

class IMessageBeforeDelete(IObjectEvent):
    """An event signalling that a message is going to be deleted
    """



from Acquisition import aq_parent
from Products.CMFCore.utils import getToolByName
from Products.IMS.interfaces import IReceivedMessageFolder, IMessage, IIMSMessage
from Products.IMS import IMSMessageFactory as _

from zope.i18n import translate
from zope.component import getSiteManager

def createMessageFolders(event):
    """ creates message-folders for the user
    """
    try:
        inst = getToolByName(event.principal, 'portal_quickinstaller')
    except:
        return
    if inst.isProductInstalled('IMS'):
        userid = event.principal.getId()
        ims = IIMSMessage(event.principal)
        ims._getMessageFolder(userid, 'received', 'ReceivedMessageFolder')
        ims._getMessageFolder(userid, 'sent', 'SentMessageFolder')

def newMessageNotify(object, event):
    """ Notifies the appropriate User if he received a new message
    """
    if IReceivedMessageFolder.providedBy(aq_parent(object)):
        try:
            mailhost = getToolByName(object, 'MailHost')
            mship = getToolByName(object, 'portal_membership')
            sender = object.getSender()
            receiver = object.getReceiver()
            portal = getSiteManager(object)
            body = object.new_message_notification(name=mship.getMemberInfo(receiver).get('fullname', object.getReceiver()),
                                                   sender=mship.getMemberInfo(sender).get('fullname', object.getSender()),
                                                   site=portal.getProperty('title', ''),
                                                   siteurl=getToolByName(object, 'portal_url')(),
                                                   url=object.absolute_url(),
                                                   emailfrom=portal.getProperty('email_from_name', ''),
                                                   message=object)
            mailhost.secureSend(body,
                                mto=mship.getMemberById(receiver).getProperty('email'),
                                mfrom='%s <%s>' % (portal.getProperty('email_from_name', ''), portal.getProperty('email_from_address', '')),
                                subject=translate(_(u'mail_subject', default=u'You have received a new message'), context=object.REQUEST),
                                charset='utf-8')
        except:
            # no mailserver
            pass

def handleMessageDelete(event):
    """ Set reference between possible reply and replyTo of message to be deleted
    """
    if IMessage.providedBy(event.object):
        replyTo = event.object.getReplyTo()
        if replyTo is not None:
            reply = event.object.getBackReferences(relationship='replyTo')
            if len(reply) > 0:
                reply[0].setReplyTo(replyTo)


"""Message folders (received, sent)
"""

from zope.interface import implements
from zope.i18n import translate

from Products.CMFCore.permissions import ModifyPortalContent, DeleteObjects

from Products.Archetypes.public import *

from Products.ATContentTypes.content.base import ATCTMixin

from Products.IMS.config import PROJECTNAME
from Products.IMS.interfaces import IMessageFolder, IReceivedMessageFolder, ISentMessageFolder
from Products.IMS import IMSMessageFactory as _

MessageFolderSchema = BaseFolderMixin.schema.copy()
MessageFolderSchema.delField('title')

class BaseMessageFolder(BaseFolderMixin, ATCTMixin):
    """ A folder holding messages
    """
    implements(IMessageFolder)

    _at_rename_after_creation = False
    schema = BaseFolderMixin.schema.copy()
    
class ReceivedMessageFolder(BaseMessageFolder):
    """ The folder holding received messages
    """
    implements(IReceivedMessageFolder)
    
    portal_type = meta_type = "ReceivedMessageFolder"
    
    def Title(self):
        try:
            return translate(_('title_receivedmessagefolder', default=u'Received Messages'), context=self.REQUEST)
        except:
            return u'Received Messages'
    
class SentMessageFolder(BaseMessageFolder):
    """ The folder holding sent messages
    """
    implements(ISentMessageFolder)
    
    portal_type = meta_type = "SentMessageFolder"
    
    def Title(self):
        try:
            return translate(_('title_sentmessagefolder', default=u'Sent Messages'), context=self.REQUEST)
        except:
            return u'Sent Messages'
    
registerType(ReceivedMessageFolder, PROJECTNAME)
registerType(SentMessageFolder, PROJECTNAME)
    


"""Message content type
"""

from AccessControl import ClassSecurityInfo

from zope.interface import implements
from zope.i18n import translate
from zope.event import notify

from Products.CMFCore.permissions import ModifyPortalContent, View

from Products.Archetypes.public import *

from Products.ATContentTypes.content.base import ATCTMixin

from Products.IMS.config import PROJECTNAME
from Products.IMS.interfaces import IMessage, IIMSMessage, ISentMessage, IReceivedMessage
from Products.IMS import IMSMessageFactory as _
from Products.IMS.events import MessageBeforeDelete

MessageSchema = BaseContent.schema.copy() + Schema((
                                                        
    StringField(
        name='sender',
        required=1,
        widget=StringWidget(
            label=_('label_sender', default=u'Sent by'),
            visible=0,
        ),
    ),
                                                        
    LinesField(
        name='receiver',
        required=1,
        widget=TextAreaWidget(
            label=_('label_receiver', default=u'Received by'),
            visible=0,
        ),
    ),
    
    TextField(
        name='message',
        required=1,
        widget=TextAreaWidget(
            label=_('label_message', default=u'Message'),
            visible=0,
        ),
    ),
    
    ReferenceField(
        name='replyTo',
        required=0,
        allowed_types=('Message',),
        relationship='replyTo',
        widget=ReferenceWidget(
            label=_('label_replyTo', default=u'Reply to'),
            visible=0,
        ),
    ),
    
    ReferenceField(
        name='companion',
        required=0,
        allowed_types=('Message',),
        relationship='companion',
        widget=ReferenceWidget(
            label=_('label_companion', default=u'Companio'),
            visible=0,
        ),
    ),
    
))

for field in ('creators','allowDiscussion','contributors','location','subject','language','rights','effectiveDate','expirationDate',):
    if MessageSchema.has_key(field):
        MessageSchema[field].widget.visible = {'edit': 'invisible', 'view': 'invisible'}

class Message(BaseContent):
    """ A message
    """
    implements(IMessage)

    portal_type = meta_type = "Message"
    _at_rename_after_creation = False
    schema = MessageSchema
    forwarded = False
    replied = False
    read = False

    security = ClassSecurityInfo()

    security.declareProtected(View, 'title_or_id')
    def title_or_id(self):
        """Returns the title if it is not blank and the id otherwise.
        """
        return self.getId()
    
    def replyToMessage(self, title, message):
        """ Reply to this message
        """
        adapter = IIMSMessage(self)
        return adapter.replyToMessage(self, title, message)
    
    def forwardMessage(self, title, message, receiver):
        """ Forward this message to somebody
        """
        adapter = IIMSMessage(self)
        return adapter.forwardMessage(self, title, message, receiver)
    
    @property
    def isSent(self):
        return ISentMessage.providedBy(self)
    
    @property
    def isReceived(self):
        return IReceivedMessage.providedBy(self)
    
    @property
    def isRead(self):
        return self.read

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        notify(MessageBeforeDelete(item))
        BaseContentMixin.manage_beforeDelete(self, item, container)
    
registerType(Message, PROJECTNAME)


from Acquisition import aq_inner, aq_parent
from AccessControl import Unauthorized

from zope.component import getMultiAdapter

from zope.interface import Interface
from zope import schema
from zope.formlib import form

from Products.statusmessages.interfaces import IStatusMessage

from Products.Five.formlib import formbase
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from Products.CMFCore.utils import getToolByName

from plone.memoize.instance import memoize

from Products.IMS.config import UseSystem, SendMessageToAny
from Products.IMS.interfaces import IIMSMessage, IMessage, IReceivedMessageFolder, ISentMessageFolder, IReceivedMessage, ISentMessage
from Products.IMS import IMSMessageFactory as _

def checkOwnership(context):
    owner = context.getWrappedOwner()
    mship = getToolByName(context, 'portal_membership')
    auth = mship.getAuthenticatedMember()
    if not owner.getId() == auth.getId():
        raise Unauthorized

class MessageFolderView(BrowserView):
    """view of a message-folder
    """

    template = ViewPageTemplateFile('templates/message_folder.pt')

    def __call__(self):
        self.request.set('disable_border', True)

        checkOwnership(self.context)

        return self.template()

    @property
    @memoize
    def sent(self):
        return ISentMessageFolder.providedBy(self.context)

    @property
    @memoize
    def received(self):
        return IReceivedMessageFolder.providedBy(self.context)

    @property
    @memoize
    def hasAnyPerm(self):
        mship = getToolByName(self.context, 'portal_membership')
        return mship.checkPermission(SendMessageToAny, self.context)

    @memoize
    def getMessages(self):
        ploneview = getMultiAdapter((self.context, self.request), name='plone')
        catalog = getToolByName(self.context, 'portal_catalog')
        mship = getToolByName(self.context, 'portal_membership')
        results = catalog(object_provides=IMessage.__identifier__, path={'query': '/'.join(self.context.getPhysicalPath())}, sort_on='created', sort_order='reverse')
        return [{'title': message.Title,
                 'message': message.getMessage,
                 'sender': message.getSender and mship.getMemberInfo(message.getSender) or message.getSender,
                 'receiver': [mship.getMemberInfo(receiver) or receiver for receiver in message.getReceiver],
                 'read': message.read,
                 'replied': message.replied,
                 'forwarded': message.forwarded,
                 'created': ploneview.toLocalizedTime(message.created, 1),
                 'uid': message.UID,
                 'url': {'base': message.getURL(),
                         'forward': '%s/%s' % (message.getURL(), 'forward'),
                         'reply': '%s/%s' % (message.getURL(), 'reply')}} for message in results]

class MessageMarkReadView(BrowserView):
    """view to mark messages as read
    """

    def __call__(self):
        self.request.set('disable_border', True)

        checkOwnership(self.context)

        catalog = getToolByName(self.context, 'portal_catalog')
        results = catalog(object_provides=IMessage.__identifier__, path={'query': '/'.join(self.context.getPhysicalPath())}, UID=self.request.get('uids', []))
        for message in results:
            message = message.getObject()
            message.read = True
            message.reindexObject()

        return self.request.RESPONSE.redirect(self.context.absolute_url())

class MessageDeleteView(MessageFolderView):
    """view to delete messages (after confirmation)
    """

    template = ViewPageTemplateFile('templates/messages_delete.pt')

    def __call__(self):
        self.request.set('disable_border', True)

        checkOwnership(self.context)

        if self.request.get('delete_confirm', None) is not None and self.request.get('paths', None) is not None:
            return self.context.restrictedTraverse('%s/folder_delete' % '/'.join(self.context.getPhysicalPath()))()
        if not self.request.get('uids', []):
            return self.request.RESPONSE.redirect(self.context.absolute_url())

        return self.template()

    @memoize
    def getMessages(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        mship = getToolByName(self.context, 'portal_membership')
        base = '/'.join(self.context.getPhysicalPath())
        results = catalog(object_provides=IMessage.__identifier__, path={'query': base}, UID=self.request.get('uids', []), sort_on='Date', sort_order='reverse')
        return [{'title': message.Title,
                 'message': message.getMessage,
                 'sender': message.getSender and mship.getMemberInfo(message.getSender) or message.getSender,
                 'receiver': [mship.getMemberInfo(receiver) or receiver for receiver in message.getReceiver],
                 'url': '%s/%s' % (base, message.getId)} for message in results]

class MessageView(BrowserView):
    """view of a message
    """

    template = ViewPageTemplateFile('templates/message.pt')

    def __call__(self):
        self.request.set('disable_border', True)

        checkOwnership(self.context)

        if not self.context.read:
            self.context.read = True
            self.context.reindexObject()

        context_state = getMultiAdapter((self.context, self.request),
                                        name=u'plone_context_state')

        self.message_actions = context_state.actions(category='message')

        plone_utils = getToolByName(self.context, 'plone_utils')
        self.getIconFor = plone_utils.getIconFor

        return self.template()

    @property
    @memoize
    def receiver(self):
        mship = getToolByName(self.context, 'portal_membership')
        return [mship.getMemberInfo(receiver) for receiver in self.context.getReceiver()]

    @property
    @memoize
    def sender(self):
        mship = getToolByName(self.context, 'portal_membership')
        return self.context.getSender() and mship.getMemberInfo(self.context.getSender()) or self.context.getSender()

    @property
    @memoize
    def date(self):
        ploneview = getMultiAdapter((self.context, self.request), name='plone')
        return ploneview.toLocalizedTime(self.context.CreationDate(), 1)

    def _getMessageValues(self, message):
        ploneview = getMultiAdapter((self.context, self.request), name='plone')
        mship = getToolByName(self.context, 'portal_membership')
        return {'title': message.Title(),
                'message': message.getMessage(),
                'sender': message.getSender() and mship.getMemberInfo(message.getSender()) or message.getSender(),
                'receiver': [mship.getMemberInfo(receiver) or receiver for receiver in message.getReceiver()],
                'created': ploneview.toLocalizedTime(message.CreationDate(), 1),
                'received': IReceivedMessage.providedBy(message),
                'sent': ISentMessage.providedBy(message)}

    @memoize
    def getMessageThread(self):
        replyTo = self.context.getReplyTo()
        thread = []
        while replyTo is not None:
            thread.append(self._getMessageValues(replyTo))
            replyTo = replyTo.getReplyTo()
        return thread

class INewMessageForm(Interface):
    """Define the fields of our form
    """

    receiver = schema.Choice(title=_(u"To"),
                             required=True,
                             source='ims.members')

    subject = schema.TextLine(title=_(u"Subject"),
                              required=True)

    message = schema.Text(title=_(u"Message"),
                          required=True)

class NewMessageForm(formbase.PageForm):
    form_fields = form.FormFields(INewMessageForm)
    label = _(u"New Message")

    def __call__(self):
        self.request.set('disable_border', True)

        self.form_fields.get('receiver').field.default = None
        self.form_fields.get('receiver').field.readonly = False
        self.form_fields.get('subject').field.default = None
        self.form_fields.get('message').field.default = None

        if self.request.get('receiver', None) is not None:
            self.form_fields.get('receiver').field.default = self.request.get('receiver', None)

        if self.request.get('subject', None) is not None:
            self.form_fields.get('subject').field.default = self.request.get('subject', None).decode('utf-8')

        return super(NewMessageForm, self).__call__()

    @form.action(_(u"send"))
    def action_send(self, action, data):
        """Send the message
        """
        return self._sendMessage(action, data)

    def _sendMessage(self, action, data):
        imsmessage = IIMSMessage(self.context)

        message = imsmessage.sendMessage(data['subject'], data['message'], data['receiver'])
        if message is not None:
            IStatusMessage(self.request).addStatusMessage(_(u"Your message has been sent successfully"), type='info')
            return self.request.response.redirect(message.absolute_url())
        else:
            IStatusMessage(self.request).addStatusMessage(_(u"Sending your message failed"), type='error')
            return self.request.response.redirect(self.context.absolute_url())

    @form.action(_(u"cancel"),validator=lambda *args, **kwargs: {})
    def action_cancel(self, action, data):
        """Cancel message-creation
        """
        return self.request.response.redirect(self.context.absolute_url())

class NewContextMessageForm(NewMessageForm):
    form_fields = form.FormFields(INewMessageForm)

    def __call__(self):
        self.request.set('disable_border', True)

        self.form_fields.get('receiver').field.default = self.context.getId()
        self.form_fields.get('receiver').field.readonly = True
        self.form_fields.get('subject').field.default = None
        self.form_fields.get('message').field.default = None

        if self.request.get('subject', None) is not None:
            self.form_fields.get('subject').field.default = self.request.get('subject', None).decode('utf-8')

        return super(NewMessageForm, self).__call__()

    @form.action(_(u"send"))
    def action_send(self, action, data):
        """Send the message
        """
        data['receiver'] = self.context.getId()
        return self._sendMessage(action, data)

class ReplyMessageForm(formbase.PageForm):
    form_fields = form.FormFields(INewMessageForm)
    label = _(u"Reply to Message")

    template = ViewPageTemplateFile('templates/message_form.pt')

    def __call__(self):
        self.request.set('disable_border', True)

        checkOwnership(self.context)

        self.form_fields.get('subject').field.default = ('Re: %s' % self.context.Title().decode('utf-8'))
        self.form_fields.get('receiver').field.default = self.context.getSender()
        self.form_fields.get('receiver').field.readonly = True
        self.form_fields.get('message').field.default = None

        return super(ReplyMessageForm, self).__call__()

    @form.action(_(u"reply"))
    def action_reply(self, action, data):
        """Reply to Message
        """
        message = self.context.replyToMessage(self.request.form.get('subject', data['subject']), data['message'])
        if message is not None:
            IStatusMessage(self.request).addStatusMessage(_(u"Your message has been sent successfully"), type='info')
            return self.request.response.redirect(message.absolute_url())
        else:
            IStatusMessage(self.request).addStatusMessage(_(u"Sending your message failed"), type='error')
            return self.request.response.redirect(self.context.absolute_url())

    @form.action(_(u"cancel"),validator=lambda *args, **kwargs: {})
    def action_cancel(self, action, data):
        """Cancel message-creation
        """
        return self.request.response.redirect(self.context.absolute_url())

    @memoize
    def getMessageThread(self):
        messageview = getMultiAdapter((self.context, self.request), name='view')
        thread = messageview.getMessageThread()
        thread.insert(0, messageview._getMessageValues(self.context))
        return thread

class ForwardMessageForm(formbase.PageForm):
    form_fields = form.FormFields(INewMessageForm)
    label = _(u"Forward Message")

    template = ViewPageTemplateFile('templates/message_form.pt')

    def __call__(self):
        self.request.set('disable_border', True)

        checkOwnership(self.context)

        self.form_fields.get('receiver').field.default = None
        self.form_fields.get('receiver').field.readonly = False

        self.form_fields.get('subject').field.default = ('Fw: %s' % self.context.Title().decode('utf-8'))
        if self.request.get('receiver', None) is not None:
            self.form_fields.get('receiver').field.default = self.request.get('receiver', None)
        self.form_fields.get('message').field.default = self.context.getMessage().decode('utf-8')

        return super(ForwardMessageForm, self).__call__()

    @form.action(_(u"forward"))
    def action_forward(self, action, data):
        """Forward Message
        """
        message = self.context.forwardMessage(self.request.form.get('subject', data['subject']), data['message'], data['receiver'])
        if message is not None:
            IStatusMessage(self.request).addStatusMessage(_(u"Your message has been sent successfully"), type='info')
            return self.request.response.redirect(message.absolute_url())
        else:
            IStatusMessage(self.request).addStatusMessage(_(u"Sending your message failed"), type='error')
            return self.request.response.redirect(self.context.absolute_url())

    @form.action(_(u"cancel"),validator=lambda *args, **kwargs: {})
    def action_cancel(self, action, data):
        """Cancel message-creation
        """
        return self.request.response.redirect(self.context.absolute_url())


from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from Products.CMFCore.utils import getToolByName
from zope.component import getSiteManager, getMultiAdapter

def members(context):
    if context is None:
        context = getSiteManager()
    mship = getToolByName(context, 'portal_membership')
    filter = getToolByName(context, 'portal_properties').site_properties.getProperty('ims_member_filter', ())
    portal_state = getMultiAdapter((context, context.REQUEST), name=u'plone_portal_state')
    member = portal_state.member()
    userid = member.getId()
    return SimpleVocabulary([SimpleTerm(m, m, mship.getMemberInfo(m).get('fullname', m).decode('UTF-8'))
                             for m in mship.listMemberIds() 
                             if not m in filter and not m == userid])


from zope.interface import implements
from zope.component.interfaces import ObjectEvent

from Products.IMS import interfaces

class MessageBeforeDelete(ObjectEvent):
    """An event signalling that a message is going to be deleted
    """
    implements(interfaces.IMessageBeforeDelete)


"""Common configuration constants
"""

from AccessControl import ModuleSecurityInfo
from Products.CMFCore.permissions import setDefaultRoles

security = ModuleSecurityInfo('Products.IMS.config')

PROJECTNAME = "IMS"

security.declarePublic('UseSystem')
UseSystem = 'IMS: Use System'
setDefaultRoles(UseSystem, ('Member',))

security.declarePublic('SendMessage')
SendMessage = 'IMS: Send Message'
setDefaultRoles(SendMessage, ('Member',))

security.declarePublic('SendMessageToAny')
SendMessageToAny = 'IMS: Send Message to Any'
setDefaultRoles(SendMessageToAny, ('Manager',))

security.declarePublic('SendMessageToMany')
SendMessageToMany = 'IMS: Send Message to Many'
setDefaultRoles(SendMessageToMany, ('Manager',))

product_globals = globals()